/*
    Title:    LCD library
    Author:   Volker Oth, 4bit mode added by Kurt Stevens
    Date:     5/1999
    Purpose:  print text on an lcd text display
    needed
    Software: AVR-GCC to compile, AVA to assemble and link
    needed
    Hardware: ATS90S8515/4414/Mega on STK200/STK300 board
              LCD text display with 8bit microcontroller connection.
    Note:     To contact me, mail to
                  volkeroth(at)gmx.de

    When using the io mode, you can't use the LCD-connection on
    STK200/300 board because the "Enable" pin is not directly connected.
    On the other hand, you can't use the memory mapped mode on devices
    which don't have support for external SRAM. AFAIK only the 4414, 8515
    and Megas support memory mapped mode.

    Memory mapped mode uses the following pins:

       DATA = PORTA
       R S  = PORTC BIT 6
       R/W  = PORTD BIT 6
      (E    = PORTC BIT 7)

    IO mode can use any pins.    
*/


#include <io.h>
#include "global.h"
#include "timer.h"
#include "lcd.h"

static u08 lcd_x, lcd_y;


/*************************************************************/
/********************** LOCAL FUNCTIONS **********************/
/*************************************************************/

#if LCD_IO_MODE
static void lcd_write(u08 data,u08 rs) 
{
    if (rs) sbi(LCD_RS_PORT, LCD_RS_PIN);
    else    cbi(LCD_RS_PORT, LCD_RS_PIN);
    /* all data pins high (inactive) */
    outp(LCD_PORT_MASK, LCD_DATA_PORT);
    /* configure data pins as output */
    outp(LCD_PORT_MASK, DDR(LCD_DATA_PORT));
    /* write mode */
    cbi(LCD_RW_PORT, LCD_RW_PIN);
    /* write (high) byte of command/data */
    outp(data, LCD_DATA_PORT);
    /* toggle enable line */
    lcd_e_high();
    lcd_e_low();
#if ! LCD_8BIT_MODE
    /* write low byte of command/data */
    outp((data<<4), LCD_DATA_PORT);
    /* toggle enable line */
    lcd_e_high();
    lcd_e_low();
#endif
}
#else
#define lcd_write(d,rs) if (rs) *(volatile u08*)(LCD_IO_DATA) = d; else *(volatile u08*)(LCD_IO_FUNCTION) = d;
/*#define lcd_write(d,rs) outp( (d), (rs) ? LCD_IO_DATA : LCD_IO_FUNCTION )*/
/* rs==0 -> write instruction to LCD_IO_FUNCTION */
/* rs==1 -> write data to LCD_IO_DATA */
#endif


#if LCD_IO_MODE
static u08 lcd_read(u08 rs) 
{
    register u08 data;

    if (rs) sbi(LCD_RS_PORT, LCD_RS_PIN);
    else    cbi(LCD_RS_PORT, LCD_RS_PIN);
    /* configure data pins as input */
    outp(~LCD_PORT_MASK, DDR(LCD_DATA_PORT));
    /* read mode */
    sbi(LCD_RW_PORT, LCD_RW_PIN);
    /* toggle enable line */
    lcd_e_high();
    /* read command/data */
    data = inp(PIN(LCD_DATA_PORT));
    lcd_e_low();
#if ! LCD_8BIT_MODE
    /* toggle enable line */
    lcd_e_high();
    data |= (inp(PIN(LCD_DATA_PORT))>>4);
    lcd_e_low();
#endif
    return data;
}
#else
#define lcd_read(rs) (rs) ? *(volatile u08*)(LCD_IO_DATA) : *(volatile u08*)(LCD_IO_FUNCTION)
/*#define lcd_read(rs) inp( (rs) ? LCD_IO_DATA : LCD_IO_FUNCTION ) */
/* rs==0 -> read instruction from LCD_IO_FUNCTION */
/* rs==1 -> read data from LCD_IO_DATA */
#endif


static void lcd_waitbusy(void)
/* loops while lcd is busy */
{
    while (lcd_read(0) & (1<<LCD_BUSY)) {}
}


static void lcd_newline(void)
/* goto start of next line */
{
    lcd_x = 0;
    if (++lcd_y >= LCD_LINES)
        lcd_y = 0;
}


static void lcd_goto(void)
/* goto position (lcd_x,lcd_y) */
{
    lcd_command((1<<LCD_DDRAM)+LCD_LINE_LENGTH*lcd_y+lcd_x);
}


/*************************************************************/
/********************* PUBLIC FUNCTIONS **********************/
/*************************************************************/

void lcd_command(u08 cmd)
/* send commando <cmd> to LCD */
{
    lcd_waitbusy();
    lcd_write(cmd, 0);
}


void lcd_gotoxy(u08 x, u08 y)
/* goto position (x,y) */
{
    lcd_x = x; lcd_y = y;
    lcd_goto();
}


void lcd_clrscr(void)
/* clear lcd */
{
    lcd_x = lcd_y = 0;
    lcd_command(1<<LCD_CLR);
    /*delay(1640);*/
}


void lcd_home(void)
/* set cursor to home position */
{
    lcd_x = lcd_y = 0;
    lcd_command(1<<LCD_HOME);
    /*delay(1640);*/
}


void lcd_putchar(u08 data)
/* print character to current cursor position */
{
    lcd_waitbusy();
    if (data=='\n') {
        lcd_newline();
        lcd_goto();
    }
    else {
        if (++lcd_x >= LCD_LINE_LENGTH) {
            lcd_newline();
            lcd_goto();
        }
        lcd_write(data, 1);
    }
}


void lcd_init(u08 cursor, u08 fnc)
/* cursor:   0 = off, 2 = on, 3 = blinking */
/* fnc: see LCD_FUNCTION_xxx */
{
    u08 wait[] = { 250, 78, 1, 1 };
    register u08 i;

#if LCD_IO_MODE
    /* configure control line pins as output */
    sbi(DDR(LCD_E_PORT),  LCD_E_PIN);
    sbi(DDR(LCD_RS_PORT), LCD_RS_PIN);
    sbi(DDR(LCD_RW_PORT), LCD_RW_PIN);
#if LCD_HW_ENABLE
    /* configure RD line as output and low */
    sbi(DDR(PORTD), 7);
    cbi(PORTD,      7); 
#else
    /* set enable line high */
    sbi(LCD_E_PORT, LCD_E_PIN);
#endif    
#else
    /* enable external SRAM (memory mapped lcd) and wait states */
    outp((1<<SRE)|(1<<SRW), MCUCR);
#endif

    fnc |= (1<<LCD_FUNCTION);
    /* reset lcd */
    for (i=0; i<4; i++) {
        delay(wait[i]<<6);                     /* 16ms, 5ms, 64us, 64us */
        lcd_write(fnc, 0);                     /* reset function */    
    }
    lcd_command(1<<LCD_ON);
    lcd_clrscr();
    lcd_command(LCD_MODE_DEFAULT);
    lcd_command((1<<LCD_ON)|(1<<LCD_ON_DISPLAY)|cursor);
} 


void lcd_puts(char s[])
/* print string on lcd (no auto linefeed) */
{
    register u08 *c;

    if (!(c=s)) return;

    while (*c) {
        lcd_putchar(*c);    
        c++;
    }
}
                             
